@@ -29,7 +29,7 @@
 #include "seahorse-gconf.h"
 #include "seahorse-gpgmex.h"
 #include "seahorse-agent.h"
-#include "seahorse-agent-secmem.h"
+#include "seahorse-secure-memory.h"
 
 /*
  * Implementation of the password cache. Note that only passwords
@@ -127,7 +127,7 @@ destroy_cache_item (gpointer data)
             g_free (it->desc);
 
         if (it->pass)
-            secmem_free (it->pass);
+            seahorse_secure_memory_free (it->pass);
 
         g_chunk_free (it, g_memory);
     }
@@ -321,7 +321,7 @@ void
 seahorse_agent_cache_set (const gchar *id, const gchar *pass,
                           gboolean encode, gboolean lock)
 {
-    int len;
+    int len, c;
     gboolean cache;
     sa_cache_t *it;
 
@@ -353,22 +353,24 @@ seahorse_agent_cache_set (const gchar *id, const gchar *pass,
     /* Work with the password */
 
     if (it->pass)
-        secmem_free (it->pass);
+        seahorse_secure_memory_free (it->pass);
 
     len = strlen (pass);
 
     if (encode) {
-        it->pass = (gchar *) secmem_malloc (sizeof (gchar *) * ((len * 2) + 1));
+        c = sizeof (gchar *) * ((len * 2) + 1);
+        it->pass = (gchar *) seahorse_secure_memory_malloc (c);
         if (!it->pass) {
             g_critical ("out of secure memory");
             return;
         }
 
+        memset (it->pass, 0, c);
         encode_password (it->pass, pass);
     }
 
     else {
-        it->pass = (gchar *) secmem_malloc (sizeof (gchar) * (len + 1));
+        it->pass = (gchar *) seahorse_secure_memory_malloc (sizeof (gchar) * (len + 1));
         if (!it->pass) {
             g_critical ("out of secure memory");
             return;
