@@ -27,10 +27,11 @@
 #include <glade/glade-xml.h>
 
 #include "eggtrayicon.h"
+#include "seahorse-pgp-key.h"
 #include "seahorse-agent.h"
 #include "seahorse-context.h"
 #include "seahorse-widget.h"
-#include "seahorse-agent-secmem.h"
+#include "seahorse-secure-memory.h"
 
 /* 
  * Implements tray icon, menu and status window. Tray icon ideas 
@@ -151,9 +152,9 @@ window_show ()
     }
 
     if (!g_sctx)
-        g_sctx = seahorse_context_new ();
-
-    g_window = seahorse_widget_new ("agent-cache", g_sctx);
+        g_sctx = seahorse_context_new (TRUE, SKEY_PGP);
+    
+    g_window = seahorse_widget_new ("agent-cache");
 
     w = glade_xml_get_widget (g_window->xml, g_window->name);
 
@@ -172,7 +173,7 @@ window_show ()
                                    G_CALLBACK (clear_clicked), g_window);
 
     /* The secure memory warning */
-    if (!secmem_have ()) {
+    if (!seahorse_secure_memory_have ()) {
         GdkColor color = { 0, 0xffff, 0, 0 };
         w = glade_xml_get_widget (g_window->xml, "insecure_label");
         gtk_widget_modify_fg (w, GTK_STATE_NORMAL, &color);
@@ -200,6 +201,18 @@ on_show_window_activate (GtkWidget *item, gpointer data)
     window_show ();
 }
 
+static void
+on_settings_activate (GtkWidget *item, gpointer data)
+{
+    GError *err = NULL;
+    g_spawn_command_line_async ("seahorse-preferences --cache", &err);
+    
+    if (err != NULL) {
+        g_warning ("couldn't execute seahorse-preferences: %s", err->message);
+        g_error_free (err);
+    }
+}
+
 /* Called when icon destroyed */
 static void
 tray_destroyed (GtkWidget *widget, void *data)
@@ -227,6 +240,8 @@ tray_clicked (GtkWidget *button, GdkEventButton *event, void *data)
                                        G_CALLBACK (on_clear_cache_activate), NULL);
         glade_xml_signal_connect_data (xml, "on_show_window_activate",
                                        G_CALLBACK (on_show_window_activate), NULL);
+        glade_xml_signal_connect_data (xml, "on_settings_activate",
+                                       G_CALLBACK (on_settings_activate), NULL);
 
         gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
                         event->button, gtk_get_current_event_time ());
