@@ -26,6 +26,23 @@
 #include "seahorse-marshal.h"
 #include "seahorse-operation.h"
 
+/* Override the DEBUG_OPERATION_ENABLE switch here */
+/* #define DEBUG_OPERATION_ENABLE 0 */
+
+#ifndef DEBUG_OPERATION_ENABLE
+#if _DEBUG
+#define DEBUG_OPERATION_ENABLE 1
+#else
+#define DEBUG_OPERATION_ENABLE 0
+#endif
+#endif
+
+#if DEBUG_OPERATION_ENABLE
+#define DEBUG_OPERATION(x) g_printerr x
+#else
+#define DEBUG_OPERATION(x) 
+#endif
+
 enum {
     DONE,
     PROGRESS,
@@ -360,7 +377,7 @@ seahorse_multi_operation_cancel (SeahorseOperation *operation)
 {
     SeahorseMultiOperation *mop;
     
-    g_return_if_fail (SEAHORSE_IS_MULTI_OPERATION (operation));
+    g_assert (SEAHORSE_IS_MULTI_OPERATION (operation));
     mop = SEAHORSE_MULTI_OPERATION (operation);
 
     seahorse_operation_list_cancel (mop->operations);
@@ -381,21 +398,18 @@ static void
 multi_operation_progress (SeahorseOperation *operation, const gchar *message, 
                           gdouble fract, SeahorseMultiOperation *mop)
 {
-	GSList *list;
+    GSList *list;
     
-    g_return_if_fail (SEAHORSE_IS_MULTI_OPERATION (mop));
-    g_return_if_fail (SEAHORSE_IS_OPERATION (operation));  
+    g_assert (SEAHORSE_IS_MULTI_OPERATION (mop));
+    g_assert (SEAHORSE_IS_OPERATION (operation));  
 	
-    g_assert (mop->operations);
-	list = mop->operations;
+    list = mop->operations;
     
     /* One or two operations, simple */
     if (g_slist_length (list) <= 1) {
 
-#ifdef _DEBUG 
-        g_printerr ("[multi-operation 0x%08X] single progress: %s %d/%d\n", (guint)mop, 
-            seahorse_operation_get_details (operation), operation->current, operation->total);
-#endif
+        DEBUG_OPERATION (("[multi-operation 0x%08X] single progress: %s %d/%d\n", (guint)mop, 
+            seahorse_operation_get_details (operation), operation->current, operation->total));
         
         seahorse_operation_mark_progress (SEAHORSE_OPERATION (mop), 
                             seahorse_operation_get_details (operation), 
@@ -433,10 +447,8 @@ multi_operation_progress (SeahorseOperation *operation, const gchar *message,
 		    }
         }		
 
-#ifdef _DEBUG 
-        g_printerr ("[multi-operation 0x%08X] multi progress: %s %d/%d\n", (guint)mop, 
-                    message, (gint)current, (gint)total);
-#endif
+        DEBUG_OPERATION (("[multi-operation 0x%08X] multi progress: %s %d/%d\n", (guint)mop, 
+                    message, (gint)current, (gint)total));
         
         seahorse_operation_mark_progress (SEAHORSE_OPERATION (mop), message,
 	    								  current, total);
@@ -449,8 +461,8 @@ multi_operation_done (SeahorseOperation *op, SeahorseMultiOperation *mop)
     GSList *l;
     gboolean done = TRUE;
     
-    g_return_if_fail (SEAHORSE_IS_MULTI_OPERATION (mop));
-    g_return_if_fail (SEAHORSE_IS_OPERATION (op));  
+    g_assert (SEAHORSE_IS_MULTI_OPERATION (mop));
+    g_assert (SEAHORSE_IS_OPERATION (op));  
   
     g_signal_handlers_disconnect_by_func (op, multi_operation_done, mop);
     g_signal_handlers_disconnect_by_func (op, multi_operation_progress, mop);
@@ -458,10 +470,8 @@ multi_operation_done (SeahorseOperation *op, SeahorseMultiOperation *mop)
     if (!seahorse_operation_is_successful (op) && !SEAHORSE_OPERATION (mop)->error)
         seahorse_operation_copy_error (op, &(SEAHORSE_OPERATION (mop)->error));
 
-#ifdef _DEBUG 
-    g_printerr ("[multi-operation 0x%08X] part complete (%d): 0x%08X/%s\n", (guint)mop, 
-                g_slist_length (mop->operations), (guint)op, seahorse_operation_get_details (op));
-#endif
+    DEBUG_OPERATION (("[multi-operation 0x%08X] part complete (%d): 0x%08X/%s\n", (guint)mop, 
+                g_slist_length (mop->operations), (guint)op, seahorse_operation_get_details (op)));
         
     /* Are we done with all of them? */
     for (l = mop->operations; l; l = g_slist_next (l)) {
@@ -475,9 +485,7 @@ multi_operation_done (SeahorseOperation *op, SeahorseMultiOperation *mop)
         return;
     }
 
-#ifdef _DEBUG 
-    g_printerr ("[multi-operation 0x%08X] complete\n", (guint)mop);
-#endif
+    DEBUG_OPERATION (("[multi-operation 0x%08X] complete\n", (guint)mop));
 
     /* Remove all the listeners */
     for (l = mop->operations; l; l = g_slist_next (l)) {
@@ -492,21 +500,17 @@ multi_operation_done (SeahorseOperation *op, SeahorseMultiOperation *mop)
 }
 
 void                     
-seahorse_multi_operation_add (SeahorseMultiOperation* mop, SeahorseOperation *op)
+seahorse_multi_operation_take (SeahorseMultiOperation* mop, SeahorseOperation *op)
 {
     g_return_if_fail (SEAHORSE_IS_MULTI_OPERATION (mop));
     g_return_if_fail (SEAHORSE_IS_OPERATION (op));
     
     if(mop->operations == NULL) {
-#ifdef _DEBUG 
-        g_printerr ("[multi-operation 0x%08X] start\n", (guint)mop);
-#endif        
+        DEBUG_OPERATION (("[multi-operation 0x%08X] start\n", (guint)mop));
         seahorse_operation_mark_start (SEAHORSE_OPERATION (mop));
     }
     
-#ifdef _DEBUG
-    g_printerr ("[multi-operation 0x%08X] adding part: 0x%08X\n", (guint)mop, (guint)op);
-#endif
+    DEBUG_OPERATION (("[multi-operation 0x%08X] adding part: 0x%08X\n", (guint)mop, (guint)op));
     
     mop->operations = seahorse_operation_list_add (mop->operations, op);
     
