@@ -20,113 +20,51 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <gnome.h>
+#include "config.h"
+#include <sys/wait.h>
+#include <unistd.h>
 #include <time.h>
 #include <stdio.h>
-#include <sys/wait.h>
+#include <unistd.h>
 
+#include <gnome.h>
 #include <libgnomevfs/gnome-vfs.h>
 
+#ifdef WITH_SHARING
+#include <avahi-glib/glib-watch.h>
+#include <avahi-glib/glib-malloc.h>
+#endif 
+
 #include "seahorse-gpgmex.h"
 #include "seahorse-util.h"
 #include "seahorse-key.h"
+#include "seahorse-pgp-key.h"
 #include "seahorse-gconf.h"
 #include "seahorse-vfs-data.h"
 
-typedef struct _SeahorsePGPHeader {
-    const gchar *header;
-    const gchar *footer;
-    SeahorseTextType type;
-} SeahorsePGPHeader;    
-
-static const SeahorsePGPHeader seahorse_pgp_headers[] = {
-    { 
-        "-----BEGIN PGP MESSAGE-----", 
-        "-----END PGP MESSAGE-----", 
-        SEAHORSE_TEXT_TYPE_MESSAGE 
-    }, 
-    {
-        "-----BEGIN PGP SIGNED MESSAGE-----",
-        "-----END PGP SIGNATURE-----",
-        SEAHORSE_TEXT_TYPE_SIGNED
-    }, 
-    {
-        "-----BEGIN PGP PUBLIC KEY BLOCK-----",
-        "-----END PGP PUBLIC KEY BLOCK-----",
-        SEAHORSE_TEXT_TYPE_KEY
-    }, 
-    {
-        "-----BEGIN PGP PRIVATE KEY BLOCK-----",
-        "-----END PGP PRIVATE KEY BLOCK-----",
-        SEAHORSE_TEXT_TYPE_KEY
-    }
-};
-
 static const gchar *bad_filename_chars = "/\\<>|";
 
 void
-seahorse_util_show_error (GtkWindow *parent, const gchar *message)
+seahorse_util_show_error (GtkWindow *parent, const gchar *heading, const gchar *message)
 {
 	GtkWidget *error;
-	g_return_if_fail (!g_str_equal (message, ""));
+    gchar *t;
+    
+	g_return_if_fail (message);
+    
+    if (heading)
+        t = g_strconcat("<big><b>", heading, "</b></big>\n\n", message, NULL);
+    else
+        t = g_strdup (message);
 	
 	error = gtk_message_dialog_new_with_markup (parent, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
-		GTK_BUTTONS_CLOSE, message);
+		GTK_BUTTONS_CLOSE, t);
+    g_free (t);
 	
 	gtk_dialog_run (GTK_DIALOG (error));
 	gtk_widget_destroy (error);
 }
 
-gchar*
-seahorse_util_get_text_view_text (GtkTextView *view)
-{
-	GtkTextBuffer *buffer;
-	GtkTextIter start;                                                      
-        GtkTextIter end;
-	gchar *text;
-	
-        g_return_val_if_fail (view != NULL, "");	
-	
-	buffer = gtk_text_view_get_buffer (view);                                                
-        gtk_text_buffer_get_bounds (buffer, &start, &end);                      
-        text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);   
-        return text;
-}
-
-void
-seahorse_util_set_text_view_string (GtkTextView *view, GString *string)
-{
-        GtkTextBuffer *buffer;
-	g_return_if_fail (view != NULL && string != NULL);
-	
-	buffer = gtk_text_view_get_buffer (view);
-	gtk_text_buffer_set_text (buffer, string->str, string->len);
-}
-
-/** 
- * seahorse_util_get_date_string:
- * @time: Time value to parse
- *
- * Creates a string representation of @time.
- *
- * Returns: A string representing @time
- **/
-gchar*
-seahorse_util_get_date_string (const time_t time)
-{
-    GDate *created_date;
-    gchar *created_string;
-    
-    if (time == 0)
-        return NULL;
-    
-    created_date = g_date_new ();
-    g_date_set_time (created_date, time);
-    created_string = g_new (gchar, 11);
-    g_date_strftime (created_string, 11, _("%Y-%m-%d"), created_date);
-    return created_string;
-}
-
 /**
  * seahorse_util_handle_gpgme:
  * @err: Error value
@@ -137,29 +75,25 @@ void
 seahorse_util_handle_gpgme (gpgme_error_t err, const char* desc, ...)
 {
     gchar *t = NULL;
-	
-	switch (gpgme_err_code(err)) {
-	case GPG_ERR_CANCELED:
-	case GPG_ERR_NO_ERROR:
-	case GPG_ERR_ECANCELED:
-		return;
-	default:
-		break;
-	}
-  
+
+    switch (gpgme_err_code(err)) {
+    case GPG_ERR_CANCELED:
+    case GPG_ERR_NO_ERROR:
+    case GPG_ERR_ECANCELED:
+        return;
+    default: 
+        break;
+    }
+
     va_list ap;
     va_start(ap, desc);
   
-    if (desc) {
-        gchar *x = g_strdup_vprintf (desc, ap);
-        t = g_strconcat("<big><b>", x, "</b></big>\n\n", gpgme_strerror (err), NULL);
-    } else {
-        t = g_strdup (gpgme_strerror (err));
-    }
-    
+    if (desc) 
+        t = g_strdup_vprintf (desc, ap);
+
     va_end(ap);
         
-    seahorse_util_show_error (NULL, t);
+    seahorse_util_show_error (NULL, t, gpgme_strerror (err));
     g_free(t);
 }
 
@@ -167,6 +101,7 @@ void
 seahorse_util_handle_error (GError* err, const char* desc, ...)
 {
     gchar *t = NULL;
+    
     va_list ap;
     
     if(!err)
@@ -174,20 +109,14 @@ seahorse_util_handle_error (GError* err, const char* desc, ...)
 
     va_start(ap, desc);
   
-    if (desc) {
-        gchar *x = g_strdup_vprintf (desc, ap);
-        t = g_strconcat("<big><b>", x, "</b></big>",
-                err->message ? "\n\n" : NULL, err->message, NULL);
-    } else if (err->message) {
-        t = g_strdup (err->message);
-    }
+    if (desc)
+        t = g_strdup_vprintf (desc, ap);
     
     va_end(ap);        
     
-    if(t != NULL)
-        seahorse_util_show_error (NULL, t);
-
+    seahorse_util_show_error (NULL, t, err->message ? err->message : "");
     g_free(t);
+    
     g_clear_error (&err);
 }    
 
@@ -204,6 +133,19 @@ seahorse_util_gpgme_error_domain ()
     return q;
 }
 
+/**
+ * seahorse_util_error_domain
+ * Returns: The GError domain for generic seahorse errors
+ **/
+GQuark
+seahorse_util_error_domain ()
+{
+    static GQuark q = 0;
+    if(q == 0)
+        q = g_quark_from_static_string ("seahorse-error");
+    return q;
+}
+
 /**
  * seahorse_util_gpgme_to_error
  * @gerr GPGME error
@@ -218,6 +160,56 @@ seahorse_util_gpgme_to_error(gpgme_error_t gerr, GError** err)
                             "%s", gpgme_strerror(gerr));
 }
 
+/** 
+ * seahorse_util_get_date_string:
+ * @time: Time value to parse
+ *
+ * Creates a string representation of @time.
+ *
+ * Returns: A string representing @time
+ **/
+gchar*
+seahorse_util_get_date_string (const time_t time)
+{
+	GDate *created_date;
+	gchar *created_string;
+	
+	if (time == 0)
+		return "0";
+	
+	created_date = g_date_new ();
+	g_date_set_time (created_date, time);
+	created_string = g_new (gchar, 11);
+	g_date_strftime (created_string, 11, _("%Y-%m-%d"), created_date);
+	return created_string;
+}
+
+gchar*
+seahorse_util_get_text_view_text (GtkTextView *view)
+{
+	GtkTextBuffer *buffer;
+	GtkTextIter start;                                                      
+    GtkTextIter end;
+	gchar *text;
+	
+    g_return_val_if_fail (view != NULL, "");	
+	
+	buffer = gtk_text_view_get_buffer (view);                                                
+    gtk_text_buffer_get_bounds (buffer, &start, &end);                      
+    text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);   
+    return text;
+}
+
+void
+seahorse_util_set_text_view_string (GtkTextView *view, GString *string)
+{
+    GtkTextBuffer *buffer;
+	g_return_if_fail (view != NULL && string != NULL);
+	
+	buffer = gtk_text_view_get_buffer (view);
+	gtk_text_buffer_set_text (buffer, string->str, string->len);
+}
+
 /**
  * seahorse_util_write_data_to_file:
  * @path: Path of file to write to
@@ -238,11 +230,7 @@ seahorse_util_write_data_to_file (const gchar *path, gpgme_data_t data,
 	gchar *buffer;
 	gint nread;
    
-    /* 
-     * TODO: gpgme_data_seek doesn't work for us right now
-     * probably because of different off_t sizes 
-     */
-    gpgme_data_rewind (data);
+    gpgme_data_seek (data, 0, SEEK_SET);
 	
     file = seahorse_vfs_data_create (path, SEAHORSE_VFS_WRITE, &err);
     if (file != NULL) {
@@ -283,11 +271,7 @@ seahorse_util_write_data_to_text (gpgme_data_t data, gboolean release)
     guint nread = 0;
 	GString *string;
 
-    /* 
-     * TODO: gpgme_data_seek doesn't work for us right now
-     * probably because of different off_t sizes 
-     */
-    gpgme_data_rewind (data);
+    gpgme_data_seek (data, 0, SEEK_SET);
 
 	string = g_string_new ("");
 	buffer = g_new (gchar, size);
@@ -414,57 +398,6 @@ seahorse_util_printf_fd (int fd, const char* fmt, ...)
     return ret;
 }
 
-/**
- * seahorse_util_detect_text
- * @text: The text to process
- * @len: Length of the text or -1 for null-terminated
- * @start: Returns the start of detected area (pass NULL if not interested)
- * @end: Returns the end of the detected area (pass NULL if not interested)
- * 
- * Auto detects what kind of PGP crypted data a given block is.
- * 
- * Returns: The type
- **/
-SeahorseTextType    
-seahorse_util_detect_text (const gchar *text, gint len, const gchar **start, 
-                            const gchar **end)
-{
-    const SeahorsePGPHeader *header;
-    const gchar *pos = NULL;
-    const gchar *t;
-    int i;
-    
-    if (len == -1)
-        len = strlen (text);
-    
-    /* Find the first of the headers */
-    for (i = 0; i < (sizeof (seahorse_pgp_headers) / sizeof (seahorse_pgp_headers[0])); i++) {
-        t = g_strstr_len (text, len, seahorse_pgp_headers[i].header);
-        if (t != NULL) {
-            if (pos == NULL || (t < pos)) {
-                header = &(seahorse_pgp_headers[i]);
-                pos = t;
-            }
-        }
-    }
-    
-    if (pos != NULL) {
-        
-        if (start)
-            *start = pos;
-        
-        /* Find the end of that block */
-        t = g_strstr_len (pos, len - (pos - text), header->footer);
-        if (t != NULL && end)
-            *end = t + strlen(header->footer);
-        else if (end)
-            *end = NULL;
-            
-        return header->type;
-    }
-    
-    return SEAHORSE_TEXT_TYPE_NONE;
-}
 
 gchar*      
 seahorse_util_filename_for_keys (GList *keys)
@@ -477,9 +410,9 @@ seahorse_util_filename_for_keys (GList *keys)
     if (g_list_length (keys) == 1) {
         g_return_val_if_fail (SEAHORSE_IS_KEY (keys->data), NULL);
         skey = SEAHORSE_KEY (keys->data);
-        t = seahorse_key_get_userid_name (skey, 0);
+        t = seahorse_key_get_simple_name (skey);
         if (t == NULL) {
-            t = g_strdup (seahorse_key_get_id (skey->key));
+            t = g_strdup (seahorse_key_get_keyid (skey));
             if (strlen (t) > 8)
                 t[8] = 0;
         }
@@ -799,8 +732,9 @@ seahorse_util_uris_package (const gchar* package, const char** uris)
         return FALSE;
     }
     
-    if (!(WIFEXITED (status) && WEXITSTATUS (status) == 0)) {
-        seahorse_util_show_error (NULL, _("The file-roller process did not complete successfully"));
+    if(!(WIFEXITED(status) && WEXITSTATUS(status) == 0)) {
+        seahorse_util_show_error(NULL, _("Couldn't package files"), 
+                                 _("The file-roller process did not complete successfully"));
         return FALSE;
     }
     
@@ -867,6 +801,25 @@ seahorse_util_chooser_show_key_files (GtkWidget *dialog)
     gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);       
 }
     
+void
+seahorse_util_chooser_show_jpeg_files (GtkWidget *dialog)
+{
+	GtkFileFilter* filter = gtk_file_filter_new ();
+     
+    /* Filter for JPEG files, this is meant to be 
+     * used when searching for images to add as a photoid*/
+     
+    gtk_file_filter_set_name (filter, _("All JPEG files"));
+    gtk_file_filter_add_mime_type (filter, "image/jpeg");
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);    
+    gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
+
+    filter = gtk_file_filter_new ();
+    gtk_file_filter_set_name (filter, _("All files"));
+    gtk_file_filter_add_pattern (filter, "*");    
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
+}   
+
 void
 seahorse_util_chooser_show_archive_files (GtkWidget *dialog)
 {
@@ -999,15 +952,14 @@ seahorse_util_check_suffix (const gchar *path, SeahorseSuffix suffix)
  * @prompt: Overwrite prompt text
  *
  * Constructs a new path for a file based on @path plus a suffix determined by
- * @suffix and the ASCII Armor setting of @ctx. If ASCII Armor is enabled, the
- * suffix will be '.asc'. Otherwise the suffix will be '.pgp' if @suffix is
- * %SEAHORSE_CRYPT_SUFFIX or '.sig' if @suffix is %SEAHORSE_SIG_SUFFIX.
+ * @suffix. If ASCII Armor is enabled, the suffix will be '.asc'. Otherwise the 
+ * suffix will be '.pgp' if @suffix is %SEAHORSE_CRYPT_SUFFIX or '.sig' if 
+ * @suffix is %SEAHORSE_SIG_SUFFIX.
  *
  * Returns: A new path with the suffix appended to @path. NULL if prompt cancelled
  **/
 gchar*
-seahorse_util_add_suffix (gpgme_ctx_t ctx, const gchar *path, 
-                          SeahorseSuffix suffix, const gchar *prompt)
+seahorse_util_add_suffix (const gchar *path, SeahorseSuffix suffix, const gchar *prompt)
 {
     GtkWidget *dialog;
     const gchar *ext;
@@ -1056,9 +1008,8 @@ seahorse_util_remove_suffix (const gchar *path, const gchar *prompt)
     GtkWidget *dialog;
     gchar *uri;
     gchar *t;
-    
-    g_return_val_if_fail (path != NULL, NULL);
-    uri =  g_strndup (path, strlen (path) - 4);
+
+	uri =  g_strndup (path, strlen (path) - 4);
    
     if (prompt && uri && seahorse_util_uri_exists (uri)) {
             
@@ -1098,6 +1049,8 @@ seahorse_util_strvec_dup (const gchar** vec)
     return ret;
 }
 
+/* TODO: This function no longer makes sense once we 
+   have multiple key and encryption types */
 gpgme_key_t* 
 seahorse_util_keylist_to_keys (GList *keys)
 {
@@ -1107,8 +1060,8 @@ seahorse_util_keylist_to_keys (GList *keys)
     recips = g_new0 (gpgme_key_t, g_list_length (keys) + 1);
     
     for (i = 0; keys != NULL; keys = g_list_next (keys), i++) {
-        g_return_val_if_fail (SEAHORSE_IS_KEY (keys->data), recips);
-        recips[i] = SEAHORSE_KEY (keys->data)->key;
+        g_return_val_if_fail (SEAHORSE_IS_PGP_KEY (keys->data), recips);
+        recips[i] = SEAHORSE_PGP_KEY (keys->data)->pubkey;
         gpgmex_key_ref (recips[i]);
     }
     
@@ -1129,8 +1082,8 @@ sort_keys_by_source (SeahorseKey *k1, SeahorseKey *k2)
 {
     SeahorseKeySource *sk1, *sk2;
     
-    g_return_val_if_fail (SEAHORSE_IS_KEY (k1), 0);
-    g_return_val_if_fail (SEAHORSE_IS_KEY (k2), 0);
+    g_assert (SEAHORSE_IS_KEY (k1));
+    g_assert (SEAHORSE_IS_KEY (k2));
     
     sk1 = seahorse_key_get_source (k1);
     sk2 = seahorse_key_get_source (k2);
@@ -1189,6 +1142,33 @@ seahorse_util_string_equals (const gchar *s1, const gchar *s2)
     return g_str_equal (s1, s2);
 }
 
+gchar*
+seahorse_util_string_up_first (const gchar *orig)
+{
+    gchar *t, *t2, *ret;
+    
+    if (g_utf8_validate (orig, -1, NULL)) {
+        
+        t = g_utf8_find_next_char (orig, NULL); 
+        if (t != NULL) {
+            t2 = g_utf8_strup (orig, t - orig);
+            ret = g_strdup_printf ("%s%s", t2, t);
+            g_free (t2);
+            
+        /* Can't find first UTF8 char */
+        } else {
+            ret = g_strdup (orig);
+        }
+    
+    /* Just use ASCII functions when not UTF8 */        
+    } else {
+        ret = g_strdup (orig);
+        g_ascii_toupper (ret[0]);
+    }
+    
+    return ret;
+}
+    
 void        
 seahorse_util_string_lower (gchar *s)
 {
@@ -1230,3 +1210,49 @@ seahorse_util_string_slist_equal (GSList *l1, GSList *l2)
     
     return !l1 && !l2;   
 }
+
+/* -----------------------------------------------------------------------------
+ * DNS-SD Stuff 
+ */
+
+#ifdef WITH_SHARING
+
+/* 
+ * We have this stuff here because it needs to:
+ *  - Be usable from libseahorse and anything within libseahorse
+ *  - Be initialized properly only once per process. 
+ */
+
+#include <avahi-glib/glib-watch.h>
+#include <avahi-glib/glib-malloc.h>
+
+static AvahiGLibPoll *avahi_poll = NULL;
+
+static void
+free_avahi ()
+{
+    if (avahi_poll)
+        avahi_glib_poll_free (avahi_poll);
+    avahi_poll = NULL;
+}
+
+const AvahiPoll*
+seahorse_util_dns_sd_get_poll ()
+{
+    if (!avahi_poll) {
+        
+        avahi_set_allocator (avahi_glib_allocator ());
+        
+        avahi_poll = avahi_glib_poll_new (NULL, G_PRIORITY_DEFAULT);
+        if (!avahi_poll) {
+            g_warning ("couldn't initialize avahi glib poll integration");
+            return NULL;
+        }
+        
+        g_atexit (free_avahi);
+    }
+    
+    return avahi_glib_poll_get (avahi_poll);
+}
+
+#endif
