@@ -19,6 +19,10 @@
  * 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+ 
+/**
+ * A bunch of miscellaneous utility functions.
+ */
 
 #ifndef __SEAHORSE_UTIL_H__
 #define __SEAHORSE_UTIL_H__
@@ -27,21 +31,20 @@
 #include <time.h>
 #include <gpgme.h>
 
+#include "config.h"
 #include "seahorse-context.h"
 
+#ifdef WITH_SHARING
+#include <avahi-client/client.h>
+const AvahiPoll* seahorse_util_dns_sd_get_poll ();
+#endif
+
 typedef enum {
 	SEAHORSE_CRYPT_SUFFIX,
 	SEAHORSE_SIG_SUFFIX,
 	SEAHORSE_ASC_SUFFIX
 } SeahorseSuffix;
 
-typedef enum {
-    SEAHORSE_TEXT_TYPE_NONE,
-    SEAHORSE_TEXT_TYPE_KEY,
-    SEAHORSE_TEXT_TYPE_MESSAGE,
-    SEAHORSE_TEXT_TYPE_SIGNED
-} SeahorseTextType;
-
 #define SEAHORSE_EXT_ASC ".asc"
 #define SEAHORSE_EXT_SIG ".sig"
 #define SEAHORSE_EXT_PGP ".pgp"
@@ -49,14 +52,20 @@ typedef enum {
 
 gchar*		seahorse_util_get_date_string		(const time_t		time);
 
-#define SEAHORSE_GPGME_ERROR  (seahorse_util_gpgme_error_domain())
-GQuark      seahorse_util_gpgme_error_domain();
+#define SEAHORSE_GPGME_ERROR  (seahorse_util_gpgme_error_domain ())
+
+GQuark      seahorse_util_gpgme_error_domain ();
+
+#define SEAHORSE_ERROR  (seahorse_util_error_domain ())
+
+GQuark      seahorse_util_error_domain ();
 
 void        seahorse_util_gpgme_to_error        (gpgme_error_t gerr, 
                                                  GError** err);
 
 void        seahorse_util_show_error            (GtkWindow          *parent,
-                                                 const char*        msg);
+                                                 const gchar        *heading,
+                                                 const gchar        *message);
                                                  
 void        seahorse_util_handle_gpgme          (gpgme_error_t      err,
                                                  const gchar*       desc, ...);
@@ -64,21 +73,23 @@ void        seahorse_util_handle_gpgme          (gpgme_error_t      err,
 void        seahorse_util_handle_error          (GError*            err,
                                                  const char*        desc, ...);
                                                  
-gpgme_error_t	seahorse_util_write_data_to_file	(const gchar		*path,
-							 gpgme_data_t		data);
+gpgme_error_t	seahorse_util_write_data_to_file (const gchar       *path,
+							                     gpgme_data_t       data,
+                                                 gboolean           release);
 
-gchar*		seahorse_util_write_data_to_text	(gpgme_data_t		data);
+gchar*		seahorse_util_write_data_to_text	(gpgme_data_t       data,
+                                                 gboolean           release);
+
+guint       seahorse_util_read_data_block       (GString            *buf, 
+                                                 gpgme_data_t       data, 
+                                                 const gchar        *start, 
+                                                 const gchar*       end);
 
 gboolean    seahorse_util_print_fd          (int fd, 
                                              const char* data);
 
 gboolean    seahorse_util_printf_fd         (int fd, 
                                              const char* data, ...);
-
-SeahorseTextType  seahorse_util_detect_text (const gchar *text, 
-                                             gint len, 
-                                             const gchar **start, 
-                                             const gchar **end);
                              
 gchar*      seahorse_util_filename_for_keys (GList *keys);
                                              
@@ -106,6 +117,8 @@ GtkWidget*  seahorse_util_chooser_save_new              (const gchar *title,
 
 void        seahorse_util_chooser_show_key_files        (GtkWidget *dialog);
 
+void        seahorse_util_chooser_show_jpeg_files       (GtkWidget *dialog);
+
 void        seahorse_util_chooser_show_archive_files    (GtkWidget *dialog);
 
 void        seahorse_util_chooser_set_filename          (GtkWidget *dialog, 
@@ -118,8 +131,7 @@ gchar*      seahorse_util_chooser_save_prompt           (GtkWidget *dialog);
 gboolean	seahorse_util_check_suffix		(const gchar		*path,
                                              SeahorseSuffix     suffix);
 
-gchar*		seahorse_util_add_suffix		(gpgme_ctx_t		ctx,
-                                             const gchar        *path,
+gchar*		seahorse_util_add_suffix		(const gchar        *path,
                                              SeahorseSuffix     suffix,
                                              const gchar        *prompt);
 
@@ -138,16 +150,26 @@ void         seahorse_util_free_keys          (gpgme_key_t* keys);
 
 gboolean    seahorse_util_string_equals       (const gchar *s1, const gchar *s2);
 
+gchar*      seahorse_util_string_up_first     (const gchar *orig);
+
+void        seahorse_util_string_lower        (gchar *s);
+
 GSList*     seahorse_util_string_slist_free   (GSList *slist);
 
 GSList*     seahorse_util_string_slist_copy   (GSList *slist);
 
-gboolean    seahorse_util_string_slist_equal   (GSList *sl1, GSList *sl2);
+gboolean    seahorse_util_string_slist_equal  (GSList *sl1, GSList *sl2);
 
 #define     seahorse_util_wait_until(expr)          \
     while (!(expr)) {                               \
         g_thread_yield ();                          \
         g_main_context_iteration (NULL, FALSE);     \
-    }                                               
+    }
+
+#ifdef _DEBUG
+#define DBG_PRINT(x) g_printerr x
+#else
+#define DBG_PRINT(x)
+#endif
 
 #endif /* __SEAHORSE_UTIL_H__ */
