@@ -26,16 +26,17 @@
 
 #include "seahorse-widget.h"
 #include "config.h"
+#include "seahorse-gtkstock.h"
 
 #define STATUS "status"
 
 enum {
 	PROP_0,
-	PROP_NAME,
-	PROP_CTX
+	PROP_NAME
 };
 
 static void     class_init          (SeahorseWidgetClass    *klass);
+static void     object_init         (SeahorseWidget         *swidget);
 
 static void     object_finalize     (GObject                *gobject);
 
@@ -77,7 +78,7 @@ seahorse_widget_get_type (void)
 		static const GTypeInfo widget_info = {
 			sizeof (SeahorseWidgetClass), NULL, NULL,
 			(GClassInitFunc) class_init,
-			NULL, NULL, sizeof (SeahorseWidget), 0, NULL
+			NULL, NULL, sizeof (SeahorseWidget), 0, (GInstanceInitFunc) object_init
 		};
 		
         widget_type = g_type_register_static (G_TYPE_OBJECT, "SeahorseWidget", 
@@ -102,9 +103,20 @@ class_init (SeahorseWidgetClass *klass)
     g_object_class_install_property (gobject_class, PROP_NAME,
         g_param_spec_string ("name", "Widget name", "Name of glade file and main widget",
                              NULL, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-    g_object_class_install_property (gobject_class, PROP_CTX,
-        g_param_spec_object ("ctx", "Seahorse Context", "Current Seahorse Context to use",
-                             SEAHORSE_TYPE_CONTEXT, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+}
+
+/* Destroy widget when context is destroyed */
+static void
+context_destroyed (GtkObject *object, SeahorseWidget *swidget)
+{
+	seahorse_widget_destroy (swidget);
+}
+
+static void
+object_init (SeahorseWidget *swidget)
+{
+    g_signal_connect_after (SCTX_APP(), "destroy", 
+                G_CALLBACK (context_destroyed), swidget);
 }
 
 /* Disconnects callbacks, destroys main window widget,
@@ -124,16 +136,14 @@ object_finalize (GObject *gobject)
     		widgets = NULL;
     	}
     }
-    	
-	g_signal_handlers_disconnect_by_func (swidget->sctx, context_destroyed, swidget);
+
+	g_signal_handlers_disconnect_by_func (SCTX_APP (), context_destroyed, swidget);
     if (glade_xml_get_widget (swidget->xml, swidget->name))
-    	gtk_widget_destroy (glade_xml_get_widget (swidget->xml, swidget->name));
+	    gtk_widget_destroy (glade_xml_get_widget (swidget->xml, swidget->name));
 	
 	g_object_unref (swidget->xml);
 	swidget->xml = NULL;
 	
-	g_object_unref (swidget->sctx);
-	
 	g_free (swidget->name);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (gobject);
@@ -144,6 +154,7 @@ object_set_property (GObject *object, guint prop_id, const GValue *value, GParam
 {
 	SeahorseWidget *swidget;
     GtkWidget *w;
+    GdkPixbuf *pixbuf = NULL;
 	char *path;
 	
 	swidget = SEAHORSE_WIDGET (object);
@@ -169,17 +180,15 @@ object_set_property (GObject *object, guint prop_id, const GValue *value, GParam
             w = glade_xml_get_widget (swidget->xml, swidget->name);
             glade_xml_set_toplevel (swidget->xml, GTK_WINDOW (w));
             glade_xml_ensure_accel (swidget->xml);
-        
-            gtk_window_set_icon (GTK_WINDOW (w), gdk_pixbuf_new_from_file (PIXMAPSDIR "seahorse.png", NULL));
+            
+            pixbuf = gtk_widget_render_icon (w, 
+                                     SEAHORSE_STOCK_SEAHORSE, 
+                                     (GtkIconSize)-1, 
+                                     NULL); 
+            gtk_window_set_icon (GTK_WINDOW (w), gdk_pixbuf_copy(pixbuf));
+            g_object_unref(pixbuf);
 			break;
             
-		case PROP_CTX:
-            g_return_if_fail (swidget->sctx == NULL);
-			swidget->sctx = g_value_get_object (value);
-			g_object_ref (G_OBJECT (swidget->sctx));
-			g_signal_connect_after (swidget->sctx, "destroy",
-				G_CALLBACK (context_destroyed), swidget);
-			break;
 		default:
 			break;
 	}
@@ -195,9 +204,6 @@ object_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *
 		case PROP_NAME:
 			g_value_set_string (value, swidget->name);
 			break;
-		case PROP_CTX:
-			g_value_set_object (value, swidget->sctx);
-			break;
 		default:
 			break;
 	}
@@ -224,29 +230,19 @@ widget_delete_event (GtkWidget *widget, GdkEvent *event, SeahorseWidget *swidget
     return FALSE; /* propogate event */
 }
 
-/* Destroy widget when context is destroyed */
-static void
-context_destroyed (GtkObject *object, SeahorseWidget *swidget)
-{
-	seahorse_widget_destroy (swidget);
-}
-
 /**
  * seahorse_widget_new:
  * @name: Name of widget, filename part of glade file, and name of main window
- * @sctx: #SeahorseContext
  *
  * Creates a new #SeahorseWidget.
  *
  * Returns: The new #SeahorseWidget, or NULL if the widget already exists
  **/
 SeahorseWidget*
-seahorse_widget_new (gchar *name, SeahorseContext *sctx)
+seahorse_widget_new (gchar *name)
 {
 	SeahorseWidget *swidget;
 	
-	g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
-	
 	/* Check if have widget hash */
 	if (widgets != NULL) {
 		swidget = g_hash_table_lookup (widgets, name);
@@ -262,7 +258,7 @@ seahorse_widget_new (gchar *name, SeahorseContext *sctx)
 		widgets = g_hash_table_new ((GHashFunc)g_str_hash, (GCompareFunc)g_str_equal);
 	
 	/* If widget doesn't already exist, create & insert into hash */
-	swidget = g_object_new (SEAHORSE_TYPE_WIDGET, "name", name, "ctx", sctx, NULL);
+	swidget = g_object_new (SEAHORSE_TYPE_WIDGET, "name", name, NULL);
 	g_hash_table_insert (widgets, g_strdup (name), swidget);
 	
 	return swidget;
@@ -271,16 +267,15 @@ seahorse_widget_new (gchar *name, SeahorseContext *sctx)
 /**
  * seahorse_widget_new:
  * @name: Name of widget, filename part of glade file, and name of main window
- * @sctx: #SeahorseContext
  *
  * Creates a new #SeahorseWidget without checking if it already exists.
  *
  * Returns: The new #SeahorseWidget
  **/
 SeahorseWidget*
-seahorse_widget_new_allow_multiple (gchar *name, SeahorseContext *sctx)
+seahorse_widget_new_allow_multiple (gchar *name)
 {
-	return g_object_new (SEAHORSE_TYPE_WIDGET, "name", name, "ctx", sctx, NULL);
+	return g_object_new (SEAHORSE_TYPE_WIDGET, "name", name,  NULL);
 }
 
 /**
@@ -295,7 +290,7 @@ seahorse_widget_show_help (SeahorseWidget *swidget)
     GError *err = NULL;
 
     if (g_str_equal (swidget->name, "key-manager"))
-        gnome_help_display (PACKAGE, NULL, &err);
+        gnome_help_display (PACKAGE, "introduction", &err);
     else
         gnome_help_display (PACKAGE, swidget->name, &err);
 
