@@ -2,6 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2003 Jacob Perkins
+ * Copyright (C) 2005 Nate Nielsen
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +20,20 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SeahorseWidget: A window created from a glade file.
+ * 
+ * - All SeahorseWidget objects are destroyed when the SeahorseContext 
+ *   goes bye-bye.
+ * - Implements fun GtkUIManager stuff.
+ * 
+ * Signals:
+ *   destroy: The window was destroyed.
+ *
+ * Properties: 
+ *   name: (gchar*) The name of the glade file to load. 
+ */
+ 
 #ifndef __SEAHORSE_WIDGET_H__
 #define __SEAHORSE_WIDGET_H__
 
@@ -44,7 +59,9 @@ struct _SeahorseWidget
 	/*< public >*/
 	GladeXML	*xml;
 	gchar		*name;
-	SeahorseContext	*sctx;
+    
+    /*< private >*/
+    GtkUIManager   *ui;
 };
 
 struct _SeahorseWidgetClass
@@ -52,14 +69,27 @@ struct _SeahorseWidgetClass
 	GObjectClass	parent_class;
 };
 
-GType           seahorse_widget_get_type ();
+GType            seahorse_widget_get_type ();
 
-SeahorseWidget*	seahorse_widget_new			(gchar			*name,
-							 SeahorseContext	*sctx);
+SeahorseWidget*  seahorse_widget_new                (gchar            *name);
 
-SeahorseWidget*	seahorse_widget_new_allow_multiple	(gchar			*name,
-							 SeahorseContext	*sctx);
+SeahorseWidget*  seahorse_widget_new_allow_multiple (gchar            *name);
 
-void		seahorse_widget_destroy			(SeahorseWidget		*swidget);
+GtkWidget*       seahorse_widget_get_top            (SeahorseWidget   *swidget);
+
+void             seahorse_widget_show               (SeahorseWidget   *swidget);
+
+void             seahorse_widget_show_help          (SeahorseWidget   *swidget);
+
+void             seahorse_widget_destroy            (SeahorseWidget   *swidget);
+
+GtkWidget*       seahorse_widget_get_ui_widget      (SeahorseWidget *swidget, 
+                                                     const gchar *path);
+
+void             seahorse_widget_add_actions        (SeahorseWidget   *swidget,
+                                                     GtkActionGroup   *actions);
+
+GtkActionGroup*  seahorse_widget_find_actions       (SeahorseWidget   *swidget, 
+                                                     const gchar *name);
 
 #endif /* __SEAHORSE_WIDGET_H__ */
