@@ -31,7 +31,6 @@
 #include "seahorse-progress.h"
 #include "seahorse-preferences.h"
 #include "seahorse-server-source.h"
-#include "seahorse-multi-source.h"
 #include "seahorse-gconf.h"
 
 static void 
@@ -44,8 +43,6 @@ sync_import_complete (SeahorseOperation *op, SeahorseKeySource *sksrc)
         seahorse_util_handle_error (err, _("Couldn't publish keys to server: %s"), 
                                     seahorse_gconf_get_string (PUBLISH_TO_KEY));
     }    
-    
-    g_object_unref (sksrc);
 }
 
 static void
@@ -61,7 +58,9 @@ sync_export_complete (SeahorseOperation *op, SeahorseKeySource *sksrc)
         g_return_if_fail (data != NULL);
 
         /* Now import it into the local key source */
-        g_object_get (sksrc, "local-source", &lsrc, NULL);
+        lsrc = seahorse_context_find_key_source (SCTX_APP (), SKEY_PGP, SKEY_LOC_LOCAL);
+        g_return_if_fail (lsrc);
+        
         if (!seahorse_key_source_import_sync (lsrc, data, &err))
             seahorse_util_handle_error (err, "Couldn't import keys");
         
@@ -76,8 +75,6 @@ sync_export_complete (SeahorseOperation *op, SeahorseKeySource *sksrc)
                                     keyserver);
         g_free (keyserver);
     }    
-    
-    g_object_unref (sksrc);
 }
 
 static void
@@ -85,7 +82,6 @@ ok_clicked (GtkButton *button, SeahorseWidget *swidget)
 {
     SeahorseKeySource *lsksrc;
     SeahorseKeySource *sksrc;
-    SeahorseContext *sctx;
     SeahorseMultiOperation *mop;
     SeahorseOperation *op;
     GError *err = NULL;
@@ -95,36 +91,35 @@ ok_clicked (GtkButton *button, SeahorseWidget *swidget)
     GList *keys;
     
     keys = (GList*)g_object_get_data (G_OBJECT (swidget), "publish-keys");
-    sctx = swidget->sctx;
+    keys = g_list_copy (keys);
     
     seahorse_widget_destroy (swidget);
     
     if (!keys)
         return;
+    
+    g_assert (SEAHORSE_IS_KEY (keys->data));
 
     /* This should be the default key source */
-    lsksrc = seahorse_context_get_key_source (swidget->sctx);
+    lsksrc = seahorse_context_find_key_source (SCTX_APP(), SKEY_PGP, SKEY_LOC_LOCAL);
     g_return_if_fail (lsksrc != NULL);
 
     mop = seahorse_multi_operation_new ();
 
     /* And now syncing keys from the servers */
     ks = seahorse_gconf_get_string_list (KEYSERVER_KEY);
-    ks = seahorse_server_source_purge_keyservers (ks);
-    
+	ks = seahorse_server_source_purge_keyservers (ks);
+	
     for (l = ks; l; l = g_slist_next (l)) {
-
-        sksrc = SEAHORSE_KEY_SOURCE (seahorse_server_source_new (lsksrc, (const gchar*)(l->data), NULL));
         
-        /* When the scheme is not supported */
-        if (!sksrc)
-            continue;
+        sksrc = seahorse_context_remote_key_source (SCTX_APP(), (const gchar*)(l->data));
+        g_return_if_fail (sksrc != NULL);
         
         op = seahorse_key_source_export (sksrc, keys, FALSE, NULL);
         g_return_if_fail (op != NULL);
 
         g_signal_connect (op, "done", G_CALLBACK (sync_export_complete), sksrc);
-        seahorse_multi_operation_add (mop, op);
+        seahorse_multi_operation_take (mop, op);
     }
     seahorse_util_string_slist_free (ks);
     
@@ -144,7 +139,9 @@ ok_clicked (GtkButton *button, SeahorseWidget *swidget)
             gerr = gpgme_data_new_from_mem (&data, exported, strlen (exported), 0);
             g_return_if_fail (GPG_IS_OK (gerr));
 
-            sksrc = SEAHORSE_KEY_SOURCE (seahorse_server_source_new (lsksrc, keyserver, NULL));
+            sksrc = seahorse_context_remote_key_source (SCTX_APP (), keyserver);
+            g_return_if_fail (sksrc != NULL);
+            
             op = seahorse_key_source_import (sksrc, data);
             g_return_if_fail (op != NULL);
             
@@ -152,23 +149,23 @@ ok_clicked (GtkButton *button, SeahorseWidget *swidget)
             g_free (exported);
             
             g_signal_connect (op, "done", G_CALLBACK (sync_import_complete), sksrc);
-            seahorse_multi_operation_add (mop, op);
+            seahorse_multi_operation_take (mop, op);
         }
         
     }
 
+    g_list_free (keys);
     g_free (keyserver);
     
     /* Show the progress window if necessary */
     if (!seahorse_operation_is_done (SEAHORSE_OPERATION (mop))) 
-        seahorse_progress_show (sctx, SEAHORSE_OPERATION (mop), 
-                                _("Syncing keys..."), FALSE);
+        seahorse_progress_show (SEAHORSE_OPERATION (mop), _("Syncing keys..."), FALSE);
 }
 
 static void
 configure_clicked (GtkButton *button, SeahorseWidget *swidget)
 {
-    seahorse_preferences_show (swidget->sctx, 3);
+    seahorse_preferences_show ("keyserver-tab");
 }
 
 static void
@@ -211,7 +208,6 @@ unhook_notification (GtkWidget *widget, gpointer data)
 
 /**
  * seahorse_keyserver_sync_show
- * @sctx: The SeahorseContext
  * @keys: The keys to synchronize
  * 
  * Shows a synchronize window.
@@ -219,7 +215,7 @@ unhook_notification (GtkWidget *widget, gpointer data)
  * Returns the new window.
  **/
 GtkWindow*
-seahorse_keyserver_sync_show (SeahorseContext *sctx, GList *keys)
+seahorse_keyserver_sync_show (GList *keys)
 {	
 	SeahorseWidget *swidget;
     GtkWindow *win;
@@ -227,7 +223,7 @@ seahorse_keyserver_sync_show (SeahorseContext *sctx, GList *keys)
     guint n, notify_id;
     gchar *t;
     
-	swidget = seahorse_widget_new ("keyserver-sync", sctx);
+	swidget = seahorse_widget_new ("keyserver-sync");
 	g_return_val_if_fail (swidget != NULL, NULL);
     
     win = GTK_WINDOW (glade_xml_get_widget (swidget->xml, swidget->name));
@@ -248,7 +244,10 @@ seahorse_keyserver_sync_show (SeahorseContext *sctx, GList *keys)
     g_signal_connect (win, "destroy", G_CALLBACK (unhook_notification), 
                       GINT_TO_POINTER (notify_id));
 
-    g_object_set_data (G_OBJECT (swidget), "publish-keys", keys);
+    keys = g_list_copy (keys);
+    g_return_val_if_fail (SEAHORSE_IS_KEY (keys->data), win);    
+    g_object_set_data_full (G_OBJECT (swidget), "publish-keys", keys, 
+                            (GDestroyNotify)g_list_free);
     
 	glade_xml_signal_connect_data (swidget->xml, "ok_clicked",
 		                           G_CALLBACK (ok_clicked), swidget);
